@@ -10,8 +10,8 @@
  * 
  * GAWK is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; either version 1, or (at your option)
+ * any later version.
  * 
  * GAWK is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +20,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with GAWK; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 /* ------------------------------ Includes ------------------------------ */
@@ -325,7 +325,7 @@ typedef struct exp_node {
 					 * the structure on many machines
 					 */
 			char *sp;
-			size_t slen;
+			short slen;
 			unsigned char sref;
 			char idx;
 		} val;
@@ -424,6 +424,7 @@ typedef struct iobuf {
 	char *end;
 	size_t size;	/* this will be determined by an fstat() call */
 	int cnt;
+	char *secbuf;
 	size_t secsiz;
 	int flag;
 #	define		IOP_IS_TTY	1
