@@ -103,7 +103,7 @@ extern int yyerrflag;
 YYSTYPE yylval, yyval;
 # define YYERRCODE 256
 
-# line 717 "awk.y"
+# line 709 "awk.y"
 
 
 struct token {
@@ -182,46 +182,44 @@ va_dcl
 {
 	va_list args;
 	char *mesg;
-	register char *bp, *cp;
+	register char *ptr, *beg;
 	char *scan;
-	char buf[120];
 
 	errcount++;
 	/* Find the current line in the input file */
 	if (lexptr) {
 		if (!thisline) {
-			for (cp=lexeme; cp != lexptr_begin && *cp != '\n'; --cp)
+			for (beg = lexeme; beg != lexptr_begin && *beg != '\n'; --beg)
 				;
-			if (*cp == '\n')
-				cp++;
-			thisline = cp;
+			if (*beg == '\n')
+				beg++;
+			thisline = beg;
 		}
 		/* NL isn't guaranteed */
-		bp = lexeme;
-		while (bp < lexend && *bp && *bp != '\n')
-			bp++;
+		ptr = lexeme;
+		while (ptr < lexend && *ptr && *ptr != '\n')
+			ptr++;
 	} else {
 		thisline = "(END OF FILE)";
-		bp = thisline + 13;
+		ptr = thisline + 13;
 	}
-	msg("%.*s", (int) (bp - thisline), thisline);
-	bp = buf;
-	cp = buf + sizeof(buf) - 24;	/* 24 more than longest msg. input */
+	msg("syntax error");
+	fprintf(stderr, "%.*s\n", (int) (ptr - thisline), thisline);
 	if (lexptr) {
 		scan = thisline;
-		while (bp < cp && scan < lexeme)
+		while (scan < lexeme)
 			if (*scan++ == '\t')
-				*bp++ = '\t';
+				putc('\t', stderr);
 			else
-				*bp++ = ' ';
-		*bp++ = '^';
-		*bp++ = ' ';
+				putc(' ', stderr);
+		putc('^', stderr);
+		putc(' ', stderr);
 	}
 	va_start(args);
 	mesg = va_arg(args, char *);
-	(void) vsprintf(bp, mesg, args);
+	vfprintf(stderr, mesg, args);
 	va_end(args);
-	msg(buf);
+	putc('\n', stderr);
 	exit(2);
 }
 
@@ -2728,34 +2726,26 @@ case 133:
 { yyval.nodeval = snode (yypvt[-1].nodeval, Node_builtin, (int) yypvt[-3].lval); } break;
 case 134:
 # line 628 "awk.y"
-{
-		if (do_lint)
-			warning("call of length without parentheses is not portable");
-		yyval.nodeval = snode ((NODE *)NULL, Node_builtin, (int) yypvt[-0].lval);
-		if (do_posix) {
-			yyerror("POSIX requires parentheses for call to `length'");
-			yyerrok;
-		}
-	  } break;
+{ yyval.nodeval = snode ((NODE *)NULL, Node_builtin, (int) yypvt[-0].lval); } break;
 case 135:
-# line 638 "awk.y"
+# line 630 "awk.y"
 {
 		yyval.nodeval = node (yypvt[-1].nodeval, Node_func_call, make_string(yypvt[-3].sval, strlen(yypvt[-3].sval)));
 	  } break;
 case 136:
-# line 642 "awk.y"
+# line 634 "awk.y"
 { yyval.nodeval = node (yypvt[-0].nodeval, Node_preincrement, (NODE *)NULL); } break;
 case 137:
-# line 644 "awk.y"
+# line 636 "awk.y"
 { yyval.nodeval = node (yypvt[-0].nodeval, Node_predecrement, (NODE *)NULL); } break;
 case 138:
-# line 646 "awk.y"
+# line 638 "awk.y"
 { yyval.nodeval = yypvt[-0].nodeval; } break;
 case 139:
-# line 648 "awk.y"
+# line 640 "awk.y"
 { yyval.nodeval = yypvt[-0].nodeval; } break;
 case 140:
-# line 651 "awk.y"
+# line 643 "awk.y"
 { if (yypvt[-0].nodeval->type == Node_val) {
 			yypvt[-0].nodeval->numbr = -(force_number(yypvt[-0].nodeval));
 			yyval.nodeval = yypvt[-0].nodeval;
@@ -2763,25 +2753,25 @@ case 140:
 			yyval.nodeval = node (yypvt[-0].nodeval, Node_unary_minus, (NODE *)NULL);
 		} break;
 case 141:
-# line 658 "awk.y"
+# line 650 "awk.y"
 { yyval.nodeval = yypvt[-0].nodeval; } break;
 case 142:
-# line 663 "awk.y"
+# line 655 "awk.y"
 { yyval.nodeval = node (yypvt[-1].nodeval, Node_postincrement, (NODE *)NULL); } break;
 case 143:
-# line 665 "awk.y"
+# line 657 "awk.y"
 { yyval.nodeval = node (yypvt[-1].nodeval, Node_postdecrement, (NODE *)NULL); } break;
 case 145:
-# line 671 "awk.y"
+# line 663 "awk.y"
 { yyval.nodeval = NULL; } break;
 case 146:
-# line 673 "awk.y"
+# line 665 "awk.y"
 { yyval.nodeval = yypvt[-0].nodeval; } break;
 case 147:
-# line 678 "awk.y"
+# line 670 "awk.y"
 { yyval.nodeval = variable(yypvt[-0].sval,1); } break;
 case 148:
-# line 680 "awk.y"
+# line 672 "awk.y"
 {
 		if (yypvt[-1].nodeval->rnode == NULL) {
 			yyval.nodeval = node (variable(yypvt[-3].sval,1), Node_subscript, yypvt[-1].nodeval->lnode);
@@ -2790,22 +2780,22 @@ case 148:
 			yyval.nodeval = node (variable(yypvt[-3].sval,1), Node_subscript, yypvt[-1].nodeval);
 		} break;
 case 149:
-# line 688 "awk.y"
+# line 680 "awk.y"
 { yyval.nodeval = node (yypvt[-0].nodeval, Node_field_spec, (NODE *)NULL); } break;
 case 150:
-# line 690 "awk.y"
+# line 682 "awk.y"
 { yyval.nodeval = node (yypvt[-0].nodeval, Node_field_spec, (NODE *)NULL); } break;
 case 152:
-# line 698 "awk.y"
+# line 690 "awk.y"
 { yyerrok; } break;
 case 153:
-# line 702 "awk.y"
+# line 694 "awk.y"
 { yyerrok; } break;
 case 156:
-# line 711 "awk.y"
+# line 703 "awk.y"
 { yyerrok; want_assign = 0; } break;
 case 157:
-# line 714 "awk.y"
+# line 706 "awk.y"
 { yyerrok; } break;
 	}
 	goto yystack;		/* reset registers in driver code */
