@@ -10,8 +10,8 @@
  * 
  * GAWK is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; either version 1, or (at your option)
+ * any later version.
  * 
  * GAWK is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +20,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with GAWK; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include "awk.h"
@@ -240,10 +240,8 @@ int flags;
 			c = *pf++;
 			if (c == '\\') {
 				c = parse_escape(&pf);
-				if (c < 0) {
-					warning("backslash at end of string");
-					c = '\\';
-				}
+				if (c < 0)
+					cant_happen();
 				*pt++ = c;
 			} else
 				*pt++ = c;
