@@ -1,4 +1,4 @@
-char *version_string = "@(#)Gnu Awk (gawk) 2.11";
+char *version_string = "@(#)Gnu Awk (gawk) 2.13";
 
 /* 1.02		fixed /= += *= etc to return the new Left Hand Side instead
 		of the Right Hand Side */
@@ -32,3 +32,10 @@ char *version_string = "@(#)Gnu Awk (gawk) 2.11";
 
 /* 2.11		Bug fix release to 2.10.  Lots of changes for portability,
 		speed, and configurability.  */
+
+/* 2.12		Lots of changes for portability, speed, and configurability.
+		Several bugs fixed.  POSIX compliance.  Removal of last set
+		of hard-wired limits.  Atari and VMS ports added. */
+
+/* 2.13		Public release of 2.12 */
+
